// URLDisplay.java
package jwiley.chp11;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.beans.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;
import java.io.BufferedInputStream;
import java.io.IOException;

/**
 * Java bean that displays URLs.  This class is very simplistic
 * it only displays text or images, regardless of the
 * content.  This is a visual Bean, so it needs to extend an AWT
 * component class
 */
public class URLDisplay extends Canvas
{
    /** value of the Display property */
    private boolean bDisplay = true;

    /** value of the URL property */
    private String strLocation = "";

    /**
     * URL object.  This only exists if the URL property
     * is properly formatted
     */
    private URL url = null;

    /**
     * Object referred to by the URL.  This only exists if the URL
     * object exists, and it pointed to a valid location
     */
    private Object curObj = null;

    /** A vector of all the listeners for our event */
    private Vector listeners = new Vector();

    /** minimum height of component. */
    int iMinHeight=200;
    /** minimum width of component. */
    int iMinWidth=300;

    /** helper to implement bound properties. */
    private PropertyChangeSupport changer = new PropertyChangeSupport(this);

    /**
     * Read the Display property
     */
    public synchronized boolean getDisplay()
    {
        return bDisplay;
    }

    /**
     * Write the Display property
     */
    public synchronized void setDisplay(boolean bNewVal)
    {
        bDisplay = bNewVal;
    }

    /**
     * Read the URL property
     */
    public synchronized String getURL()
    {
        return strLocation;
    }

    /**
     * Write the URL property
     */
    public synchronized void setURL(String newStr)
    {
        String oldValue = strLocation;
        strLocation = newStr;
        changer.firePropertyChange("URL", oldValue, newStr);
        reload();
    }

    /** Add a PropertyChangeListener. */
    public void addPropertyChangeListener(PropertyChangeListener pcl)
    {
        changer.addPropertyChangeListener(pcl);
    }

    /** Remove a PropertyChangeListener. */
    public void removePropertyChangeListener(PropertyChangeListener pcl)
    {
        changer.addPropertyChangeListener(pcl);
    }

    /**
     * Load (or reload) the object pointed to by the URL
     */
    public synchronized void reload()
    {
        curObj = "Loading...";

        try
        {
            url = new URL(strLocation);
            curObj = url.getContent();
            if (curObj instanceof BufferedInputStream)
            {
                boolean bStore = true;
                StringBuffer sb = new StringBuffer();
                Vector v = new Vector();
                while (((BufferedInputStream)curObj).available() > 0)
                {
                    char c = (char)((BufferedInputStream)curObj).read();

                    if (!bStore && c == '>')
                    {
                        bStore = true;
                        continue;
                    }
                    if (c == '<')
                    {
                        bStore = false;
                        continue;
                    }
                    if (bStore)
                    {
                        if (c == '\f' || c == '\t' || c == '\r')
                            continue;

                        if (c != '\n')
                        {
                            sb.append(c);
                        }
                        else
                        {
                            if (sb.length() > 1)
                                v.addElement(sb.toString());
                            sb = new StringBuffer();
                        }
                    }
                }
                if (sb.length() > 0)
                    v.addElement(sb.toString());

                if (v.size() > 0)
                {
                    String [] outStrings = new String[v.size()];
                    for (int i=0; i < outStrings.length; i++)
                        outStrings[i] = (String) v.elementAt(i);
                    curObj = outStrings;
                }
            }
            else if (curObj instanceof ImageProducer)
            {
                curObj = Toolkit.getDefaultToolkit().createImage((ImageProducer)curObj);
            }
        }
        catch (Exception e)
          {
            e.printStackTrace();
            curObj = null;
          }

        fireEvent(curObj);
        repaint();
    }

    public Dimension getPreferredSize()
    { return getMinimumSize(); }

    public Dimension getMinimumSize()
    { return new Dimension(iMinWidth, iMinHeight); }

    /**
     * Paint the object if we have it.  Paint an error
     * message otherwise.
     */
    public synchronized void paint(Graphics g)
    {
        int fontHeight=0;
        if (bDisplay)
        {
            if (curObj == null)
            {
                if (url == null)
                {
                    g.drawString("No URL specified", 10, 10);
                }
                else
                {
                    g.drawString("There was an error loading the URL",
                                 10, 10);
                }
            }
            else if (curObj instanceof String [])
            {
                fontHeight = (g.getFontMetrics(g.getFont())).getHeight();
                int y = 10;
                for (int i=0; i < ((String []) curObj).length; i++)
                {
                    g.drawString((String) ((String [])curObj)[i], 10, y);
                    y+= fontHeight;
                }
            }
            else if (curObj instanceof Image)
            {
                g.drawImage((Image)curObj,0,0,this);
            }
            else
            {
                g.drawString("Could not display URL", 10, 10);
            }
        }
    }

    // event support
    /**
     * Add a listener for our event
     */
    public synchronized void addURLLoadedListener(URLLoadedListener l)
    {
        listeners.addElement(l);
    }

    /**
     * Remove a listener for our event
     */
    public synchronized void removeURLLoadedListener(URLLoadedListener l)
    {
        listeners.removeElement(l);
    }

    /**
     * Fire our event to signify that we are done loading the URL
     */
    public synchronized void fireEvent(Object obj)
    {
        URLLoadedEvent event = new URLLoadedEvent(this, obj);
        for (int i = 0; i < listeners.size(); i++)
        {
            ((URLLoadedListener)listeners.elementAt(i)).URLLoaded(event);
        }
    }

    public static void main(String [] args)
    {
        // get URL from the command line
        if (args.length < 1)
        {
            System.out.println("USAGE: java URLDisplay <URL-to-load>");
            System.exit(1);
        }

        URLDisplay loader = new URLDisplay();
        loader.setURL(args[0]);
        Frame f = new Frame("URLDisplay Bean Tester.");
        f.add("Center", loader);
        f.setLocation(100,100);
        f.pack();
        f.addWindowListener(new WindowAdapter()
                            {
                                public void windowClosing(WindowEvent we)
                                { System.exit(0); }
                            });
        f.show();
    }
}
