import java.io.*;
import java.security.*;
/**
 * This class exercises the java.Security.Signature class and its support classes
 * For instructional purposes the use of the signature functions
 * was broken into three parts.
 *		Creating the a signature key set 
 *		Generate a signature
 *      Verify the sigature
 * <pre>
 * CalculateSignature signatureTest = new CalculateSignature(args[0],args[1]);
 * byte[] signature = signatureTest.Sign(args[1]);
 * boolean result = signatureTest.VerifySignature(args[1],signature);
 * </pre>
 *  @version 1.0
 *  @author A. Saganich
 *  @see java.Security.MessageDigest
 *
 */
class CalculateSignature
{
	int 				keymodulus = 1024; 
	String 				algorithm;
	KeyPair 			keys;
	Signature 			signature;

	/**
	 * Construct the object
	 */ 
	CalculateSignature(String inAlgorithm, String inFileName)
	throws FileNotFoundException, NoSuchAlgorithmException
	{
		algorithm = inAlgorithm;

		// May throw file not found.
		FileInputStream   	file = new FileInputStream(inFileName);
		KeyPairGenerator keyGenerator = KeyPairGenerator.getInstance(algorithm);
		// We could provide a byte array to SecureRandom
		// but for simplicity let it seed itself.
		keyGenerator.initialize( keymodulus, new SecureRandom());

		// Now generate a pair of keys
		keys = keyGenerator.generateKeyPair();
		signature = Signature.getInstance(algorithm);
	}
	/**
	 * Verify a signature
	 */
	public boolean VerifySignature(String inFileName, byte[] verifySignature)
	throws FileNotFoundException,IOException,InvalidKeyException,SignatureException
	{
		FileInputStream   	file = new FileInputStream(inFileName);
		byte data[] = new byte[512];
		int ctRead;
		// Get the publicKey
		PublicKey publicKey = keys.getPublic();

		// Use it to initialize (move to the VERIFY state)
		signature.initVerify(publicKey);
		while(true)
		{
			ctRead = file.read(data);
			if ( ctRead <= 0) break;
			signature.update(data);
		}
		return signature.verify(verifySignature);
	}
	
	/**
	 * Generate a signature
	 */
	public byte[] Sign(String inFileName)
	throws FileNotFoundException,
		   NoSuchAlgorithmException, IOException, InvalidKeyException, 
		   SignatureException
	{
		FileInputStream   	file = new FileInputStream(inFileName);
		byte data[] = new byte[512];
		int ctRead;
		// Get the private key and generate a signature.
		PrivateKey privateKey = keys.getPrivate();

		// Use it to initialize (move to the SIGN state)
		signature.initSign(privateKey);
		while(true)
		{
			ctRead = file.read(data);
			if ( ctRead <= 0) break;
			signature.update(data);
		}

		return signature.sign();
	}
}
/**
 *  generateSignature outputs a signature of the file given using the algoritm supplied.
 *  @version 1.0
 *  @author A. Saganich
 *
 *  @see java.Security.KeyPairGenerator
 *  @see java.Security.KeyPair
 *  @see java.Security.SecureRandom
 */

class generateSignature
{
	public static void main(String args[])
	{
		if ( args.length < 2)
		{
			System.out.println("Usage: generateSignature AlgorithmName fileToBeSigned\n");
			System.exit(0);
		}

		System.out.println("Attempting to generate and use a signature");
		System.out.println("\t          using " + args[0]);
		System.out.println("\t          on file " + args[1]);

		// Open the file first
		// It takes forever to generate a key pair and we don't want to
		// wait and then discover that the input file is not there!
		try
		{
			System.out.println("Generating a key pair please be patient this may take a while");
			CalculateSignature signatureTest = new CalculateSignature(args[0],args[1]);
			// Get the signature

			System.out.println("Generating signature from file and private key");
			byte[] signature = signatureTest.Sign(args[1]);
			System.out.println("Verifying generated signature with public key");
			boolean result = signatureTest.VerifySignature(args[1],signature);
			System.out.println("Test comparison yielded "+ result + "(Should be true).");
		}
		catch (FileNotFoundException e)
		{
			System.out.println("Cannot find file " + args[1]);
			System.exit(0);
		}
		catch (NoSuchAlgorithmException e)
		{
			System.out.println("Unknown algorithm " + args[0]);
			System.exit(0);
		}
		catch (InvalidKeyException e)
		{
			System.out.println("Error initializing signature object!");
			System.exit(0);
		}
		catch (SignatureException e)
		{
			System.out.println("Error processing signature object!");
			System.exit(0);
		}
		catch (IOException e)
		{
			System.out.println("Unexpected IO Exception!");
			System.exit(0);
		}

	}
}