import java.io.*;
import javax.crypto.*;
import java.security.*;
/**
 * This class exercises the java.Security.Cipher class and its support classes
 * For instructional purposes the use of the Cipher functions
 * was broken into three parts.
 *		Creating the a  key set 
 *		Encrypt using the key
 *      Decrypt using the key and the Cipher text.
 *		Validate that the two files match
 *
 * <pre>
 * 			CipherText cipherTest = new CipherText(Some algorithm,inputfilename);
 * 			String encyptedName = cipherTest.Encrypt(inputfilename);
 * 			String decyptedName = cipherTest.Decrypt(encyptedName);
 * 			boolean result = signatureTest.Validate(inputfilename,decyptedName);
 * </pre>
 *  @version 1.0
 *  @author A. Saganich
 *  @see java.Security.MessageDigest
 *
 */
class CipherText
{

	String 				algorithm;
	Key	 			    key;
	Cipher	 			cipher;

	/**
	 * Construct the object
	 */ 
	CipherText(String inAlgorithm, String inFileName)
	throws FileNotFoundException, NoSuchAlgorithmException, NoSuchPaddingException
	{
		algorithm = inAlgorithm;

		// May throw file not found.
		FileInputStream   	file = new FileInputStream(inFileName);
		KeyGenerator 	keyGen = null;
		try
		{
			keyGen = KeyGenerator.getInstance(inAlgorithm);
		}
		catch (NoSuchAlgorithmException e)
		{
			System.out.println("KeyGenerator.getInstance for " + inAlgorithm + " failed!");
			throw e;
		}
		// We could provide a byte array to SecureRandom
		// but for simplicity let it seed itself.

		// Now generate a pair of keys
		keyGen.init(new SecureRandom());
		key  = keyGen.generateKey();
		cipher= Cipher.getInstance(inAlgorithm);
	}
	/**
	 * Encrypt a file
	 */
	public String Encrypt(String inFileName)
	throws 
		FileNotFoundException,IOException,InvalidKeyException,SignatureException,
		KeyException,BadPaddingException,IllegalBlockSizeException
	{
		System.out.println("Initialising cipher");
		cipher.init(cipher.ENCRYPT_MODE,key);

		FileInputStream   	inFile = new FileInputStream(inFileName);
		FileOutputStream    fos = new FileOutputStream(inFileName+".ciphertext");
		CipherOutputStream	outFile = new CipherOutputStream(fos,cipher);
		byte cleartext[] = new byte[512];
		int ctRead;

		while((ctRead = inFile.read(cleartext)) > 0)
		{
			outFile.write(cleartext,0,ctRead);
		}
		outFile.flush();		
		outFile.close();
		fos.close();
		inFile.close();
		return inFileName+".ciphertext";
	}
	
	/**
	 * Decrypt a file
	 */
	public String Decrypt(String inFileName)
	throws FileNotFoundException,
		   NoSuchAlgorithmException, IOException, InvalidKeyException, 
		   SignatureException,KeyException,BadPaddingException,IllegalBlockSizeException
	{
		cipher.init(cipher.DECRYPT_MODE,key);
		FileInputStream   	fos = new FileInputStream(inFileName+".ciphertext");
		CipherInputStream   inFile = new CipherInputStream(fos,cipher);
		FileOutputStream	outFile = new FileOutputStream(inFileName+".cleartext");
		
		System.out.println("Input is from " +inFileName+".ciphertext");
		System.out.println("Output is to " +inFileName+".ciphertext");
		byte cleartext[] = new byte[512];
		int ctRead;
		// Get the private key and generate a signature.

		while( (ctRead = inFile.read(cleartext)) > 0)
		{
			outFile.write(cleartext,0,ctRead);
		}
		outFile.flush();
		outFile.close();
		inFile.close();
		fos.close();
		return inFileName+".cleartext";
	}

	public boolean CompareFiles(String fileA, String fileB)
	throws FileNotFoundException,IOException
	{
		FileInputStream   	inFileA = new FileInputStream(fileA);
		FileInputStream   	inFileB = new FileInputStream(fileB);
		byte dataA[] = new byte[512];
		byte dataB[] = new byte[512];
		int ctReadA;
		int ctReadB;
		while(true)
		{
			ctReadA = inFileA.read(dataA);
			ctReadB = inFileB.read(dataB);
			if ( ctReadA != ctReadB)
				return false;
			else if (ctReadA <= 0)
				break;
			for ( int i = 0; i < ctReadA; i++)
			{
				if (dataA[i] != dataB[i])
					return false;
			}
		}
		inFileA.close();
		inFileB.close();

		return true;
	}
}
/**
 *  generateSignature outputs a signature of the file given using the algoritm supplied.
 *  @version 1.0
 *  @author A. Saganich
 *
 *  @see java.Security.KeyPairGenerator
 *  @see java.Security.KeyPair
 *  @see java.Security.SecureRandom
 */

class tstCipher
{
	public static void main(String args[])
	{
		if ( args.length < 2)
		{
			System.out.println("Usage: tstCipher AlgorithmName fileToBeCiphered\n");
			System.exit(0);
		}

		System.out.println("Attempting to generate a key and cipher text");
		System.out.println("\t          using " + args[0]);
		System.out.println("\t          on file " + args[1]);

		// Open the file first
		// It takes forever to generate a key pair and we don't want to
		// wait and then discover that the input file is not there!
		try
		{
			System.out.println("Generating a key pair please be patient this may take a while");
			CipherText cipherTest = new CipherText(args[0],args[1]);
			// Get the signature

			System.out.println("Generating cipher from file and key");
			String encyptedName = cipherTest.Encrypt(args[1]);
			System.out.println("Decrypting " + encyptedName );
			String decyptedName = cipherTest.Decrypt(args[1]);
			System.out.println("Validating decrypted result vs original");
			boolean result = cipherTest.CompareFiles(args[1],decyptedName);
			System.out.println("Test comparison yielded "+ result + "(Should be true).");
		}
		
		catch (IllegalBlockSizeException e)
		{
			System.out.println("Unexpected IllegalBlockSizeException " + e);
			System.exit(0);
		}
		catch (KeyException e)
		{
			System.out.println("Unexpected KeyException " + e);
			System.exit(0);
		}
		catch (FileNotFoundException e)
		{
			System.out.println("Cannot find file " + args[1]);
			System.exit(0);
		}
		catch (NoSuchPaddingException e)
		{
			System.out.println("Invalid padding supplied");
			System.out.println("Check padding and rerun");
			System.exit(0);
		}
		catch (BadPaddingException e)
		{
			System.out.println("Bad padding supplied");
			System.out.println("Check padding and rerun");
			System.exit(0);
		}
		
		catch (NoSuchAlgorithmException e)
		{
			System.out.println("Unknown algorithm " + args[0]);
			System.exit(0);
		}
		catch (SignatureException e)
		{
			System.out.println("Error processing signature object!");
			System.exit(0);
		}
		catch (IOException e)
		{
			System.out.println("Unexpected IO Exception!");
			System.exit(0);
		}

	}
}