/** ApplicationShell.java */
package jwiley.chp21;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.io.*;
import java.net.*;

/** A reusable application shell that provides the basic
    framework common to all graphical applications.  Specifically
    provides a menu, menubar, status panel and mdi interface. */
public class ApplicationShell extends JFrame
{
    /** Where internal frames are added to. */
    JDesktopPane desktop;
    /** Menu bar. */
    JMenuBar mBar;
    /** Tool bar. */
    JToolBar tBar;
    /** Used to map command names to actions. */
    Hashtable commands = new Hashtable();
    /** A class which displays a status panel. */
    StatusPanel status;
    /** Text field in the second box in the status panel. */
    JTextField secondStatusBox;

    /** array of abstract actions. */
    Action [] actions = { new NewAction(this),
                          new OpenAction(),
                          new SaveAction(),
                          new ExitAction(),
                          new CopyAction(),
                          new CutAction(),
                          new PasteAction(),
                          new AboutAction(),
                          new HelpAction(), };

    /** accessor. */
    public JDesktopPane getDesktopPane() { return desktop; }

    /** No-arg constructor. */
    public ApplicationShell()
    {
        // call parent constructor
        super("Application Shell");

        // build menus
        for (int i=0; i < actions.length; i++)
            commands.put(actions[i].getValue(Action.NAME),
                         actions[i]);
        mBar = buildMenus(commands);
        this.setJMenuBar(mBar);

        // build tool bar
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout(2,2));
        tBar = buildToolBar(commands);
        contentPane.add("North", tBar);

        // build status panel
        secondStatusBox = new JTextField(10);
        secondStatusBox.setEditable(false);
        secondStatusBox.setText(" 0 items");
        status = new StatusPanel(secondStatusBox);
        contentPane.add("South", status);
        status.setText("This application is ready.");

        // add Desktop pane (for MDI)
        desktop = new JDesktopPane();
        contentPane.add("Center", desktop);

        setDesktopSize(this, 80, 80);
        centerOnScreen(this);

        // "show" the window
        this.setVisible(true);

        // shutdown adapter
        this.addWindowListener(new WindowAdapter()
                          {
                            public void windowClosing(WindowEvent we)
                            { System.exit(0); }
                          });
    }

    /** Method to size a window. */
    public static void setDesktopSize(JFrame frame, int wPerc, int hPerc)
    {
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        int iWidth = screen.width * wPerc / 100;
        int iHeight = screen.height * hPerc / 100;
        frame.setSize(iWidth, iHeight);
    }

    /** Method to center the window. */
    public static void centerOnScreen(JFrame frame)
    {
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension window = frame.getSize();
        int iCenterX = screen.width / 2;
        int iCenterY = screen.height / 2;
        frame.setLocation(iCenterX - (window.width / 2),
                          iCenterY - (window.height / 2));
    }

    /** Method to add Menus & Menuitems to the MenuBar. */
    private JMenuBar buildMenus(Hashtable commands)
    {
        JMenuBar bar = new JMenuBar();

        // Create File Menu
        JMenu fileMenu = new JMenu("File");
        addMenuItem(fileMenu,"New",commands);
        addMenuItem(fileMenu,"Open",commands);
        addMenuItem(fileMenu,"Save",commands);
        // highlight 'exiting' to reduce accidental exits
        fileMenu.add(new JSeparator());
        addMenuItem(fileMenu,"Exit",commands);

        // Create Edit Menu
        JMenu editMenu = new JMenu("Edit");
        addMenuItem(editMenu,"Copy",commands);
        addMenuItem(editMenu,"Cut", commands);
        addMenuItem(editMenu,"Paste", commands);

        // Create Help Menu
        JMenu helpMenu = new JMenu("Help");
        addMenuItem(helpMenu, "About this application", commands);
        addMenuItem(helpMenu, "Help", commands);

        bar.add(fileMenu);
        bar.add(editMenu);
        bar.add(helpMenu);
        return bar;
    }

    /** Method to add menuitems and connect the actions to the
        menubar. */
    private void addMenuItem(JMenu m, String cmd, Hashtable commands)
    {
        JMenuItem anItem = new JMenuItem(cmd);
        Action a = (Action) commands.get(cmd);
        anItem.setEnabled(a.isEnabled());
        anItem.addActionListener(a);
        m.add(anItem);
    }

    /** Overloaded method to add an existing menu item to a menu. */
    private void addMenuItem(JMenu m, JMenuItem jm, String cmd, Hashtable commands)
    {
        Action a = (Action) commands.get(cmd);
        jm.setEnabled(a.isEnabled());
        jm.addActionListener(a);
        m.add(jm);
    }

    /** Method to construct a toolbar. */
    private JToolBar buildToolBar(Hashtable commands)
    {
        JToolBar bar = new JToolBar();

        // lock it in
        bar.setFloatable(false);
        bar.setBorderPainted(true);

        // New, Open, Save, Copy, Cut, Paste
        addButton(bar, "New", commands);
        addButton(bar, "Open", commands);
        addButton(bar, "Save", commands);
        addButton(bar, "Copy", commands);
        addButton(bar, "Cut", commands);
        addButton(bar, "Paste", commands);

        return bar;
    }

    /** Method to add a button to a toolbar. */
    private void addButton(JToolBar tBar, String cmd, Hashtable commands)
    {
        URL loc = this.getClass().getResource("images" + File.separator + (cmd.toLowerCase()) + ".gif");
        ImageIcon ic = new ImageIcon(loc);
        JButton aButton = new JButton(ic);
        aButton.addActionListener((Action)commands.get(cmd));
        tBar.add(aButton);
    }

    /** Main method to invoke from the command line. */
    public static void main(String args[])
    {
        new ApplicationShell();
    }
}

/** Action class to create a new document. */
class NewAction extends AbstractAction
{
    ApplicationShell app;
    public NewAction(ApplicationShell shell)
    {
        super("New");
        putValue(SHORT_DESCRIPTION, "Create a New Document");
        app = shell;
    }

    public void actionPerformed(ActionEvent ae)
    {
        // Create a new JInternalFrame and add it to the desktop
        JInternalFrame win = new JInternalFrame("Document",true, true, true, true);
        app.desktop.add(win, JLayeredPane.DEFAULT_LAYER);
        app.status.setText("New Document created.");

        int makeCount = 1;
        win.setBounds(20*(makeCount%10), 20*(makeCount%10), 225, 150);

        app.desktop.repaint();
    }
}

/** Action class to open a document. */
class OpenAction extends AbstractAction
{
    public OpenAction()
    {
        super("Open");
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to save a document. */
class SaveAction extends AbstractAction
{
    public SaveAction()
    {
        super("Save");
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to exit the application. */
class ExitAction extends AbstractAction
{
    public ExitAction()
    {
        super("Exit");
        putValue(SHORT_DESCRIPTION, "Exit the application");
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.exit(0);
    }
}

/** Action class to cut text from a document. */
class CutAction extends AbstractAction
{
    public CutAction()
    {
        super("Cut");
        setEnabled(false);
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to copy text from a document. */
class CopyAction extends AbstractAction
{
    public CopyAction()
    {
        super("Copy");
        setEnabled(false);
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to paste text to a document. */
class PasteAction extends AbstractAction
{
    public PasteAction()
    {
        super("Paste");
        setEnabled(false);
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to get info on this application. */
class AboutAction extends AbstractAction
{
    public AboutAction()
    {
        super("About this application");
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Action class to bring up help on using this application. */
class HelpAction extends AbstractAction
{
    public HelpAction()
    {
        super("Help");
    }

    public void actionPerformed(ActionEvent ae)
    {
        System.out.println(ae);
    }
}

/** Class to create a status panel. */
class StatusPanel extends JPanel
{
    JTextField statusMessage;
    JComponent secondBox;
    JComponent thirdBox;

    public JTextField getStatusMessage() { return statusMessage; }
    public JComponent getSecondBox() { return secondBox; }
    public JComponent getThirdBox() { return thirdBox; }

    public void setText(String sText)
    {
        statusMessage.setText(sText);
    }

    public StatusPanel()
    {
        this(null,null);
    }

    public StatusPanel(JComponent secondBox)
    {
        this(secondBox, null);
    }

    public StatusPanel(JComponent second, JComponent third)
    {
        this.secondBox = second;
        this.thirdBox = third;
        this.setLayout(new BorderLayout(2,2));
        JPanel eastPanel = new JPanel(new FlowLayout(FlowLayout.LEFT,2,0));
        this.setBorder(new SoftBevelBorder(BevelBorder.RAISED));
        statusMessage = new JTextField();
        statusMessage.setEditable(false);
        statusMessage.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        this.add("Center", statusMessage);
        if (secondBox == null)
            secondBox = new SpaceCanvas();
        secondBox.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        eastPanel.add(secondBox);

        if (thirdBox == null)
            thirdBox = new SpaceCanvas();
        thirdBox.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        eastPanel.add(thirdBox);
        this.add("East", eastPanel);
    }
}

/** A component that takes up space to create empty boxes. */
class SpaceCanvas extends JComponent
{
    public Dimension getMinimumSize()
    {
        Dimension d = getParent().getSize();
        int w = 60, h = 20;
        if (d.height > h)
            h = d.height - 1;

        return new Dimension(w,h);
    }

    public Dimension getPreferredSize() { return getMinimumSize(); }
}

