// capacityMeter.java
package jwiley.chp21;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * A LightWeight, custom component.
 * This CapacityMeter is a variation of a progress meter
 * (also known as a guage).  Since it is circular it is
 * more compact than a typical progress meter.
 */
public class CapacityMeter extends Component
{
    /** Current setting of meter current/max = percentage filled. */
    private double current;
    /** Maximum value of meter setting. */
    private double maximum;
    /** Color of Wedge that indicates percentage complete. */
    private Color wedgeColor;
    /** Color of pie face. */
    private Color pieColor;
    /** Color of top shaded edge. */
    private Color pieLightEdge;
    /** Color of Bottom shaded edge. */
    private Color pieDarkEdge;
    /** Debug flag. */
    private boolean debug=false;
    /** Dimensions of control. */
    private Dimension cmDim;

    /** mutator method .*/
    public void setDebug(boolean flag) { debug = flag; }

    /** mutator method.  Also causes an update of the pie face.
        This control uses Dirty rectangle animation. */
    public void setCurrent(double curCapacity)
    {
        current = curCapacity;
        update(this.getGraphics());
    }

    /** Constructor. */
    public CapacityMeter(double fullCapacity)
    {
        this(fullCapacity, Color.gray, Color.gray.brighter(),
                           Color.gray.darker(),
                           Color.red, new Dimension(110,110));
    }

    /** Constructor that allows different colors and a custom size. */
    public CapacityMeter(double fullCapacity, Color pieFace,
                         Color pieTop, Color pieBottom, Color wedgeFace,
                         Dimension theDim)
    {
        if (debug)
            System.out.println("Invoked CapacityMeter(int fullCapacity)");
        setBackground(Color.lightGray);
        maximum = fullCapacity;
        pieColor = pieFace;
        pieLightEdge = pieTop;
        pieDarkEdge = pieBottom;
        wedgeColor = wedgeFace;
        cmDim = theDim;
    }

    /** Preferred size to tell the layout manager. */
    public Dimension getPreferredSize()
    {
        if (debug)
            System.out.println("Invoked preferredSize()");

        return cmDim;
    }

    /** minimum size for this control. */
    public Dimension getMinimumSize()
    {
        if (debug)
            System.out.println("Invoked minimumSize()");

        return cmDim;
    }

    /** How to paint the component. In our control,
        this paints the background. To avoid flicker the
        "foreground" is painted in the update() method.
        Another way to do this is to use double-buffered
        animation - see the section on Double-buffering. */
    public void paint(Graphics g)
    {
        if (debug)
            System.out.println("Invoked paint(Graphics g)");
        Rectangle r = getBounds();
        if (debug)
            System.out.println("getBounds(): " + r);

        int innerX=5, innerY=5;
        int outerX=2, outerY=2;
        int innerSubtract=9, outerSubtract=4;

        // outer will be two 180 degree wedges
        g.setColor(pieLightEdge);
        g.fillArc(outerX, outerY, r.width - outerSubtract, r.height - outerSubtract,
              226, -182);
        g.setColor(pieDarkEdge);
        g.fillArc(outerX, outerY, r.width - outerSubtract, r.height - outerSubtract,
              225, 180);


        // inner
        g.setColor(pieColor);
        g.fillOval(innerX, innerY, r.width - innerSubtract, r.height - innerSubtract);

        update(g);
    }

    /** How to update only a portion of the component. */
    public void update(Graphics g)
    {
        // dirty rectangle animation
        Rectangle r = getBounds();
        int innerX=5, innerY=5;
        int outerX=2, outerY=2;
        int innerSubtract=9, outerSubtract=4;
        if (current > 0)
        {
            // calculate percentage
            double perc = (current/maximum) * 100;

            // calculate degrees
            int degrees = (int) ((360 * perc)/(double)100);

            // wedge
            g.setColor(wedgeColor);
            g.fillArc(innerX, innerY, r.width - innerSubtract, r.height - innerSubtract,
                      270, -degrees);

        }
    }

    /** This method overrides the default implementation (in Component)
        and will force the size and shape of the component to stay circular,
        be centered in the component area (which will be affected by
        resizing the window), and always stay the size set at construction.
        You may want to comment out this method and see what happens. */
    public synchronized void setBounds(int x, int y, int w, int h)
    {
        int centerX=0, centerY=0;
        if (w > cmDim.width)
            centerX = x + ((w - cmDim.width)/2);
        else
            centerX = x;
        if (h > cmDim.height)
            centerY = y + ((h - cmDim.height)/2);
        else
            centerY = y;
        super.setBounds(centerX,centerY,cmDim.width,cmDim.height);
        if (debug)
            System.out.println("Invoked setBounds(" + x + "," + y + ","
                                                  + w + "," + h + ")");
    }

    /** Main method to invoke from JVM for unit testing. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java CapacityMeter bkgrndFile");
            System.exit(1);
        }

        BackgroundFrame tstFrame = new BackgroundFrame(args[0]);

        try { Thread.sleep(2000); } catch (Exception e) { }

        CapacityMeter cm = null;

        while ((cm = tstFrame.getCapacityMeter()) == null)
            try { Thread.sleep(2000); } catch (Exception e) { }

        for (int i=0; i < 100; i++)
        {
            cm.setCurrent(i);
            try { Thread.sleep(500); } catch (Exception e) { }
        }
    } // end of main()
} // end of class CapacityMeter

/** Frame class used in testing.  Primary purpose of the class is
    to override frame so we could draw an image in the background
    to show the transparency of lightweight components.  Of course,
    for a circular component this is essential! */
class BackgroundFrame extends Frame
{
    /** Image to draw in the background. */
    Image img;
    /** Capacity meter in this frame. */
    CapacityMeter cm;

    /** Simple adapter to shutdown the application when the
        window close box is clicked. */
    public class ShutdownAdapter extends WindowAdapter
    {
        public void windowClosing(WindowEvent we)
        {
            System.exit(0);
        }
    }

    /** accessor for capacity meter. */
    public CapacityMeter getCapacityMeter() { return cm; }

    /** Constructor. */
    public BackgroundFrame(String sBackgroundFileName)
    {
        super("Capacity Meter Test");
        // add a capacity Meter
        cm = new CapacityMeter(100);
        cm.setDebug(false);
        add("Center", cm);

        File f = new File(sBackgroundFileName);
        if (f.exists())
        {
            img = (Toolkit.getDefaultToolkit()).getImage(sBackgroundFileName);
            MediaTracker mt = new MediaTracker(this);
            mt.addImage(img,0);
            try
            {
                mt.waitForAll();
            } catch (Exception e)
              { /* do nothing - img will be null */ }
            if (mt.isErrorAny())
            {
                System.out.println("Error loading image.");
                img = null;
            }
        }
        else
            System.out.println(sBackgroundFileName + " does not exist.");

        setSize(300,300);
        setLocation(50,50);
        addWindowListener(new ShutdownAdapter());
        show();

        try { Thread.sleep(500); } catch (Exception e) { }
    }

    /** paint method. *NOTE*: the call to super.paint(g) is necessary
        to paint lightweight components in this container. */
    public void paint(Graphics g)
    {
        Dimension d = getSize();
        if (img != null)
            g.drawImage(img, 0, 0, d.width, d.height, this);
        super.paint(g);
    }

    /** avoid flicker. */
    public void update(Graphics g)
    {
        paint(g);
    }
}

