/** TocTreePanel.java */
package jwiley.chp21;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import java.util.Vector;

/**
 * Class that generates a Tree from a heirarchically formatted text
 * file.
 * @author Michael C. Daconta
 */
public class TocTreePanel extends JPanel implements TreeSelectionListener
{
    /** Debug variable. */
    public static boolean debug;

    /** Tree to generate. */
    DefaultMutableTreeNode dynamicTree;

    /** Graphical Tree Outline. */
    JTree jt;

    /** List Selection Model. */
    TreeSelectionModel tsm;

    /** Tree Model. */
    TreeModel jtm;

    /** Accessor. */
    public DefaultMutableTreeNode getDynamicTree() { return dynamicTree; }

    /** utility method to determine what level this node is at by
        counting white space characters.  The simplest way to format
        the text file is to use one tab per level. */
    private int countWhiteSpace(String sBuf)
    {
        for (int i=0; i < sBuf.length(); i++)
            if (sBuf.charAt(i) != '\t' &&
                sBuf.charAt(i) != ' ')
                return i;
        return -1; // error
    }

    /** Utility inner class to store both the position and a
        reference to the DefaultMutableTreeNode at that position. */
    class LevelNode
    {
        /** position of node. */
        public int iPosition;
        /** Tree node at that position. */
        public DefaultMutableTreeNode node;

        /** constructor. */
        LevelNode(int iPos, DefaultMutableTreeNode t)
        { iPosition = iPos; node = t; }
    }

    /** Utility method to find the parent of the current node.
        The algorithm is to start with the end of the Vector and
        find out where this node "fits".  A fit is when this
        nodes position is greater than the previous one. */
    private DefaultMutableTreeNode findParent(Vector levels, int nodePos)
    {
        int iLast = levels.size();
        // start from the bottom
        for (int i = (iLast - 1); i >= 0; i--)
        {
            LevelNode ln = (LevelNode) levels.elementAt(i);
            if (nodePos > ln.iPosition)
                return ln.node;
        }
        return null; // error
    }

    /** Utility method to parse a "Toc" file.  File where each new
        level is indicated by indenting that word.  One word per line. */
    private DefaultMutableTreeNode parseTableOfContents(FileInputStream fis) throws IOException
    {
        DefaultMutableTreeNode outNode = null;
        int iCurLevel = 0;
        Vector vLevels = new Vector();
        BufferedReader br = new BufferedReader(
                            new InputStreamReader(fis));
        String sLine = null;
        while ( (sLine = br.readLine()) != null)
        {
            if (debug) System.out.println("Line: " + sLine);

            // get position of token
            int pos = countWhiteSpace(sLine);
            DefaultMutableTreeNode aNode = new DefaultMutableTreeNode(sLine.trim());
            vLevels.addElement(new LevelNode(pos, aNode));

            if (outNode == null)
            {
                outNode = aNode;
            }
            else
            {
                // find who to add this node to
                DefaultMutableTreeNode parent = findParent(vLevels, pos);
                if (parent != null)
                {
                    parent.add(aNode);
                }
            }
        }

        return outNode;
    }

    /** Constructor. */
    public TocTreePanel(String sTocFile) throws IOException
    {
        setLayout(new BorderLayout());

        // check if file exists
        File f = new File(sTocFile);
        if (f.exists())
        {
            FileInputStream fis = new FileInputStream(sTocFile);
            dynamicTree = parseTableOfContents(fis);
            fis.close();
        }
        else
            throw new IOException(sTocFile + " does not exist.");

        if (dynamicTree != null)
        {
            // create a JTree
            jt = new JTree(dynamicTree);

            // get the Tree Model
            jtm = jt.getModel();

            // listen to the selections
            tsm = jt.getSelectionModel();
            tsm.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            tsm.addTreeSelectionListener(this);

            // create a scrollPane
	        JScrollPane scrollpane = new JScrollPane();
	        scrollpane.getViewport().add(jt);

	        add("Center", scrollpane);
        }
        else
            throw new IOException(sTocFile + " is empty.");
    }

    public void valueChanged(TreeSelectionEvent tse)
    {
        // get the path selected
        TreePath tp = tse.getPath();
        // get the leaf node of the path
        Object o = tp.getLastPathComponent();
        // for now, just print out the leaf node (a String)
        System.out.println(o);
    }

    /** Main method for unit testing. */
    public static void main(String args[])
    {
        if (args.length < 1)
        {
            System.out.println("USAGE: java TocTreePanel tocFile");
            System.exit(1);
        }

        TocTreePanel.debug = true;

        try
        {
            TocTreePanel tp = new TocTreePanel(args[0]);
        } catch (IOException ioe)
          {
            ioe.printStackTrace();
          }
        System.exit(0);
    }
}
