import java.lang.Thread;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.lang.Runnable;
import java.util.Random;

import List.ListHead;
import MyThread;

/** Class to get and store key simulation data. */
class SimData
{
    /** number of tellers on duty at the bank. */
	int numTellers;
	/** number of seconds to run the simulation. */
	int secondsToRun;

    /** constructor. */
	SimData() throws Exception
	{
		String secStr=null;
		String numStr=null;

		System.out.println(" ");
		System.out.print("Enter number of tellers on duty: ");
		System.out.flush();
		try
		{
			numStr = BankSim.dis.readLine();
		}
		catch (IOException ioe)
		{
			System.out.println(ioe.toString());
			System.exit(1);
		}

		if (numStr != null && numStr.length() > 0)
			numTellers = Integer.parseInt(numStr);
		else
			throw new Exception("Invalid number of tellers.");

		System.out.println("Time is compressed for this Sim.");
		System.out.println("One second equals one minute.");
		System.out.print("Enter number of seconds to run Sim: ");
		System.out.flush();
		try
		{
			secStr = BankSim.dis.readLine();
		}
		catch (IOException ioe)
		{
			System.out.println(ioe.toString());
			System.exit(1);
		}

		if (secStr != null && secStr.length() > 0)
			secondsToRun = Integer.parseInt(secStr);
		else
			throw new Exception("Invalid number of Seconds.");

	}

    /** accessor. */
	int getnumTellers() { return numTellers; }

    /** accessor. */
	int getsecondsToRun() { return secondsToRun; }
}

/** Thread for running the sim. */
class SimRun extends MyThread
{
    /** current sim time. */
	public long simTime;
	/** flag. */
	public boolean quitting_time;
	/** timer thread. */
	Thread clockThread;
	/** random number generator. */
	public Random dice;
	/** data for sim. */
	SimData sData;
	/** simulation statistics. */
	SimStats sStats;
	/** bank queue. */
	ListHead bankLine;

    /** calculate probably that a customer will walk in
        the door. */
	static int customerProbability(long minute_in_day)
	{
		/* uses a Parabola mapped to an 8-hour day.
		   x-axis is minutes.
		   y-axis is probability. */

		return (int) - ( ( (minute_in_day - 240) *
						   (minute_in_day - 240) ) -
						   (3 * 240 * 98) ) /
						   (3 * 240);
	}

    /** constructor. */
	SimRun(SimData theData, SimStats theStats)
	{
		simTime = 0;
		quitting_time = false;
		dice = new Random();
		sData = theData;
		sStats = theStats;
		bankLine = new ListHead(false);
	}

    /** implements runnable. */
	public void PerformSomeWork()
	{
		int duration = sData.getsecondsToRun();

		// generate tellers
		int numTellers = sData.getnumTellers();

		// update stats
		sStats.numTellers = numTellers;
		sStats.simDuration = duration;

		for (int i=0; i < numTellers; i++)
		{
				Teller aTeller = new Teller(bankLine, this, sStats);
				aTeller.start();
		}

		// kick off clock thread
		Clock aClock = new Clock(this);
		clockThread = new Thread(aClock);

		// set clock priority as the highest in group
		clockThread.setPriority(6);
		clockThread.start();

		// sleep a sec
		try { sleep(1000); } catch (Exception e) { }

		while (!quitting_time)
		{
			// generate customers based on function
			int probability = customerProbability(simTime);

			//System.out.println("Probability: " + probability);

			// roll dice
			int roll = Math.abs(dice.nextInt() % 100);

			if (roll <= probability)
			{
				System.out.println("A customer just walked in...");

				// update stats
				sStats.totalNumCustomers += 1;

				// generate customer thread
				Customer aCustomer = new Customer(bankLine);
				aCustomer.start();
			}

			// update line size stats
			if (bankLine.size() > sStats.maxLineSize)
					sStats.maxLineSize = bankLine.size();

			try
			{
				sleep(1000);
			}
			catch (Exception e) { }


			if (simTime >= duration)
					quitting_time = true;
		}
		System.out.println("Simulation complete.");
	}
}

/** clock thread. */
class Clock implements Runnable
{
	SimRun theRun;

	Clock(SimRun aRun) { theRun = aRun; }

	public void run()
	{
		while (!theRun.quitting_time)
		{
			try
			{
				Thread.sleep(1000);
				theRun.simTime++;
				if (theRun.simTime % 10 == 0)
				   System.out.println("CLOCK: " +theRun.simTime);

			}
			catch (Exception e) { }
		}
	}
}

/** abstract action class. */
abstract class Action
{
	abstract int execute();
}

/** specific action. */
class OpenAccount extends Action
{
	int execute()
	{
		System.out.println("open an account...");
		return 5;
	}
}

/** specific action. */
class CloseAccount extends Action
{
	int execute()
	{
		System.out.println("close an account...");
		return 3;
	}
}

/** specific action. */
class RobBank extends Action
{
	int execute()
	{
		System.out.println("rob the bank...");
		return 20;
	}
}

/** specific action. */
class DepositCheck extends Action
{
	int execute()
	{
		System.out.println("deposit a check...");
		return 4;
	}
}

/** specific action. */
class CashCheck extends Action
{
	int execute()
	{
		System.out.println("cash a check...");
		return 7;
	}
}

/**
 * Thread per customer.
 *  The concept of each entity in a Simulation being a
 *  separate thread has great potential for creating
 *  Autonomous, Polymorphous Entities (APE).  This would be a
 *  large breakthrough for the modeling of computer
 *  generated forces (cgf) and human intelligence collection.
 */
class Customer extends Thread
{
    /** count of all customers generated. */
	public static int customerCount = 0;
	/** unique id of this customer. */
	private int Id;
	/** bank line to join. */
	ListHead bankLine;
	/** action to perform. */
	Action myAction=null;

    /** constructor. */
	Customer()
	{
		customerCount++;
		Id = customerCount;
	}

    /** constructor. */
	Customer(ListHead theLine)
	{
		customerCount++;
		Id = customerCount;
		bankLine = theLine;
	}

	/** accessor. */
	Action getmyAction() { return myAction; }

    /** accessor. */
	int getId() { return Id; }

    /** implements runnable. */
	public void run()
	{
		/* be busy a random number of sim minutes before
		 * entering the bank line.
		 */
		Random dice = new Random();
		int busy = Math.abs(dice.nextInt() % 10);

		try { sleep(busy * 1000); } catch (Exception e) { }

		/* after this "thinking" the customer knows
		 * what they came to do.
		 */
		int decide = Math.abs(dice.nextInt() % 5);

		switch (decide)
		{
				case 0:
						myAction = new OpenAccount();
						break;
				case 1:
						myAction = new CloseAccount();
						break;
				case 2:
						myAction = new RobBank();
						break;
				case 3:
						myAction = new DepositCheck();
						break;
				case 4:
						myAction = new CashCheck();
						break;
				default:
						myAction = new OpenAccount();
						System.out.println("Unknown action.");
		}

		// now wait on line
		bankLine.addElement(this);
		System.out.println("Customer #" + Id +
						   " is now waiting on line.");

		// sleep a little to allow println
		try { sleep(2000); } catch (Exception e) { }
	}
}

/** Thread for teller. */
class Teller extends Thread
{
    /** count of all tellers. */
	public static int tellerCount = 0;
	/** unique id of this teller. */
	private int Id;
	/** bank line. */
	ListHead theLine;
	/** simulation environment. */
	SimRun sRun;
	/** sim statistics. */
	SimStats sStats;

    /** constructor. */
	Teller(ListHead aLine, SimRun aRun, SimStats inStats)
	{
		tellerCount++;
		Id = tellerCount;
		theLine = aLine;
		sRun = aRun;
		sStats = inStats;
	}

    /** implements runnable. */
	public void run()
	{
		// pause a second to allow menu println to finish
		try { sleep(1000); } catch (Exception e) { }

		// if free, pops a customer and serves the customer
		boolean stillWaiting = false;
		while (!sRun.quitting_time)
		{
			if (theLine.size() > 0)
			{
				stillWaiting = false;
				Customer currentCustomer = (Customer) theLine.removeFirst();
				System.out.print("Teller #" +
								  Id + " is helping " +
								  "Customer #" +
								  currentCustomer.getId() +
								  " to ");

				System.out.flush();
				int helping =
						(currentCustomer.getmyAction()).execute();

				// update the stats
				sStats.totalCustomersServed += 1;

				try
				{
					sleep(helping * 1000);
				}
				catch (Exception e) { }
			}
			else
			{
			   if (!stillWaiting)
					System.out.println("Teller #" + Id +
									   " is waiting to help someone.");
			   stillWaiting = true;
			}

			try { sleep(2000); } catch (Exception e) { }
		}
	}
}

/** simulation statistics. */
class SimStats
{
	public int numTellers;
	public int simDuration;
	public int totalNumCustomers;
	public int totalCustomersServed;
	public int maxLineSize;
	// exercise: maxCustomerWait, maxTellerWait

    /** constructor. */
	SimStats()
	{
		numTellers = 0;
		simDuration = 0;
		totalNumCustomers = 0;
		totalCustomersServed = 0;
		maxLineSize = 0;
	}

    /** method to print the statistics. */
	void printStats()
	{
		System.out.println(" ");
		System.out.println("<<<<< SIM STATISTICS >>>>>");
		System.out.println("# Tellers            : " + numTellers);
		System.out.println("# minutes simulated  : " + simDuration);
		System.out.println("Tot customers entered: " + totalNumCustomers);
		System.out.println("Tot customers served : " + totalCustomersServed);
		System.out.println("Maximum line size     : " + maxLineSize);
	}
}

/** Class to present a simple menu-driven interface to control
    the bank simulation. */
public class BankSim
{
	static BufferedReader dis = new BufferedReader(new InputStreamReader(System.in));

    /** main method to invoke from JVM. */
	public static void main(String args[])
	{
		boolean done = false;
		SimData sData = null;
		SimRun sRun = null;
		SimStats sStats = new SimStats();
		while (!done)
		{
			System.out.println(" ");
			System.out.println("<<<<<< Bank Sim >>>>>>");
			System.out.println("   1) Enter Simulation Data.");
			System.out.println("   2) Run Simulation.");
			System.out.println("   3) Stop Simulation.");
			System.out.println("   4) Print Simulation Stats.");
			System.out.println("   5) Exit.");
			System.out.println(" ");
			System.out.print("Enter choice: ");
			System.out.flush();
			String choiceStr=null;
			try
			{
				choiceStr = dis.readLine();
			}
			catch (IOException ioe)
			{
				System.out.println(ioe.toString());
				System.exit(1);
			}

			if (choiceStr != null && choiceStr.length() > 0)
			{
				int choice = Integer.parseInt(choiceStr);
				switch (choice)
				{
						case 1:
							try
							{
								sData = new SimData();
							}
							catch (Exception e)
							{
								System.out.println(e.toString());
								sData = null;
							}
							break;
						case 2:
							if (sData != null)
							{
								sRun = new SimRun(sData,sStats);
								sRun.start();
								System.out.println("Sim started.");
							}
							else
								System.out.println("Must enter sim data first.");
							break;
						case 3:
							if (sRun != null)
							{
								System.out.print("Stopping...");

								sRun.quitting_time = true;
								try
								{
									Thread.sleep(1000);
								}
								catch (Exception e) { }
								System.out.println("Done.");
								sRun.changeState(MyThread.STATE_STOP);
								System.out.println("Sim stopped.");
							}
							else
								System.out.println("Sim must be running to stop.");
							break;
						case 4:
							sStats.printStats();
							break;
						case 5:
							done = true;
							break;
						default:
							System.out.println("Invalid choice");
				}
			}
			else
				System.out.println("Invalid choice.");
		}
		System.exit(1);
	}
}
