
package jwiley.chp8;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;

import java.util.Date;
import java.util.StringTokenizer;
import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * This class implements a simple reminder program. It allows for input
 * of new reminders, allows the listing of existing reminders, and will
 * display a reminder when the specified date has past.
 *
 * @author Eric Monk
 * @version 1.0
 *
 */

public class SimpleReminder extends Thread
{
    BufferedReader   input;
    PrintWriter      output;
    ReminderData     reminders[];
    String           sDataFileName;

    /** SimpleReminder constructor */
    public SimpleReminder(BufferedReader input, PrintWriter output, String sDataFileName)
    {
        this.input = input;
        this.output = output;
        this.sDataFileName = sDataFileName;

        try {
            reminders = loadReminders(sDataFileName);
        } catch (Exception e)
          {
            output.println("Error loading reminder data file." +
                           "\nError: " + e.toString());
            reminders = null;
          }
    }

    /** overrides run method in class Thread */
    public void run()
    {
        output.println("Simple Reminder v1.0");
        output.println("Type 'help' for a list of options");

        reminders = checkReminders(reminders, output, sDataFileName);

        String sCommand = new String();

        while (true)
        {
            output.println();
            output.print("reminder>");
            output.flush();

            try {
                sCommand = input.readLine();
            } catch (IOException ioe)
              {
                output.println("Error in input data: " + ioe.toString());
                continue;
              }

            if (sCommand.equalsIgnoreCase("help"))
                printHelp(output);

            else if (sCommand.equalsIgnoreCase("add"))
                reminders = addReminder(reminders, input, output,
                                                    sDataFileName);

            else if (sCommand.equalsIgnoreCase("list"))
                listReminders(reminders, output);

            else if (sCommand.equalsIgnoreCase("quit"))
                System.exit(0);

            else
                output.println("Invalid command: " + sCommand);

            reminders = checkReminders(reminders, output, sDataFileName);
        }
    }

    /** will check all reminders to see if the reminder date is before
        the current date */
    ReminderData[] checkReminders(ReminderData reminders[],
                        PrintWriter output, String sDataFileName)
    {
        Date dtNow = new Date();

        if (reminders != null)
        {
            int iNumToDelete = 0;
            boolean bDeleteReminder[] = new boolean[reminders.length];

            for (int iCount = 0; iCount < reminders.length; iCount++)
            {
                if (dtNow.after(reminders[iCount].getReminderDate()))
                {
                    output.println(reminders[iCount].getReminderText());
                    bDeleteReminder[iCount] = true;
                    iNumToDelete++;
                }
                else
                    bDeleteReminder[iCount] = false;
            }

            if (iNumToDelete == 0)
                return (reminders);
            else
            {
                // collapse array
                ReminderData newReminders[] =
                        new ReminderData[reminders.length - iNumToDelete];
                int iOffset = 0;

                for (int iCount = 0; iCount < reminders.length; iCount++)
                {
                    if (bDeleteReminder[iCount])
                        iOffset++;
                    else
                        newReminders[iCount - iOffset] = reminders[iCount];
                }

                // write reminders to reminder file
                try {
                    writeRemindersToFile(newReminders, sDataFileName);
                } catch (IOException ioe)
                  {
                    output.println("Error writing reminders to file." +
                                   "\nError: " + ioe.toString());
                    return (reminders);
                  }

                return (newReminders);
            }
        }
        else
            return (reminders);
    }

    /** method that will load reminders from a file */
    ReminderData[] loadReminders(String sDataFileName)
                                    throws IOException, Exception
    {
        File fDataFile = new File(sDataFileName);
        if (!fDataFile.exists())
            return (null);

        FileInputStream fis = new FileInputStream(sDataFileName);
        ObjectInputStream oInStream = new ObjectInputStream(fis);

        int iNumReminders = oInStream.readInt();
        if (iNumReminders > 0)
        {
            ReminderData storedReminders[] =
                                    new ReminderData[iNumReminders];

            for (int iCount = 0; iCount < iNumReminders; iCount++)
            {
                storedReminders[iCount] =
                                (ReminderData) oInStream.readObject();
            }

            fis.close();
            return (storedReminders);
        }
        else
        {
            fis.close();
            return (null);
        }
    }

    /** method to print help for this program */
    void printHelp(PrintWriter output)
    {
        output.println("Simple Reminder v1.0 Help");
        output.println("\tadd\tAdd reminder");
        output.println("\thelp\tDisplay help listing");
        output.println("\tlist\tList all reminders");
        output.println("\tquit\tQuit program");
    }

    /** method to add a reminder to the current set of reminders */
    ReminderData[] addReminder(ReminderData reminders[],
                               BufferedReader input,
                               PrintWriter output,
                               String sDataFileName)
    {
        String sDate;
        String sTime;
        String sText;

        output.print("Date: ");
        output.flush();

        try {
            sDate = input.readLine();
        } catch (IOException ioe)
          {
            output.println("Error in input data: " + ioe.toString());
            return (reminders);
          }

        output.print("Time: ");
        output.flush();

        try {
            sTime = input.readLine();
        } catch (IOException ioe)
          {
            output.println("Error in input data: " + ioe.toString());
            return (reminders);
          }

        output.print("Text: ");
        output.flush();

        try {
            sText = input.readLine();
        } catch (IOException ioe)
          {
            output.println("Error in input data: " + ioe.toString());
            return (reminders);
          }

        Date dt = null;
        try {
            dt = parseDate(sDate, sTime);
        } catch (Exception e)
          {
            output.println(e.toString());
            return (reminders);
          }

        // add reminder to reminder array
        ReminderData newReminder = new ReminderData(dt, sText);
        int iNumReminders = 0;
        if (reminders != null)
            iNumReminders = reminders.length;

        ReminderData newReminders[] = new ReminderData[iNumReminders + 1];
        for (int iCount = 0; iCount < iNumReminders; iCount++)
            newReminders[iCount] = reminders[iCount];

        newReminders[newReminders.length - 1] = newReminder;

        // add reminder to reminder file
        try {
            writeRemindersToFile(newReminders, sDataFileName);
        } catch (IOException ioe)
          {
            output.println("Error writing reminder to file." +
                           "\nError: " + ioe.toString());
            return (reminders);
          }

        return (newReminders);
    }

    /** method to list all reminders in the current set */
    void listReminders(ReminderData reminders[], PrintWriter output)
    {
        if (reminders != null)
        {
            output.println("Date\t\tTime\tText");
            output.println("----\t\t----\t----");

            GregorianCalendar calendar = new GregorianCalendar();

            for (int iCount = 0; iCount < reminders.length; iCount++)
            {
                calendar.setTime(reminders[iCount].getReminderDate());
                String sDate = (calendar.get(Calendar.MONTH) + 1) +
                               "/" + calendar.get(Calendar.DAY_OF_MONTH) +
                               "/" + calendar.get(Calendar.YEAR);

                int iMinute = calendar.get(Calendar.MINUTE);
                String sMinute = "";
                if (iMinute < 10)
                    sMinute = "0" + iMinute;
                else
                    sMinute = "" + iMinute;

                String sTime = calendar.get(Calendar.HOUR_OF_DAY) +
                                                            ":" + sMinute;

                String sText = reminders[iCount].getReminderText();

                output.println(sDate + "\t" +
                               sTime + "\t" +
                               sText);
            }
        }
        else
            output.println("No reminders are set");

    }

    /** method to write an array of reminders to a file */
    void writeRemindersToFile (ReminderData reminders[],
                               String sDataFileName)
                               throws IOException
    {
        FileOutputStream fos = new FileOutputStream(sDataFileName);
        ObjectOutputStream oOutStream = new ObjectOutputStream(fos);

        if (reminders != null)
        {
            oOutStream.writeInt(reminders.length);
            for (int iCount = 0; iCount < reminders.length; iCount++)
            {
                oOutStream.writeObject(reminders[iCount]);
            }
        }

        oOutStream.flush();
        fos.close();
    }

    /** method to convert a textual date and time into a Date class */
    Date parseDate (String sDate, String sTime) throws Exception
    {
        StringTokenizer stDate = new StringTokenizer(sDate, "/");
        StringTokenizer stTime = new StringTokenizer(sTime, ":");

        int iMonth = 0;
        int iDay = 0;
        int iYear = 0;
        int iHour = 0;
        int iMinute = 0;

        try {
            iMonth = Integer.parseInt(stDate.nextToken()) - 1;
            iDay = Integer.parseInt(stDate.nextToken());
            iYear = Integer.parseInt(stDate.nextToken());
        } catch (Exception e)
          {
            throw new Exception("Invalid date format");
          }

        try {
            iHour = Integer.parseInt(stTime.nextToken());
            iMinute = Integer.parseInt(stTime.nextToken());
        } catch (Exception e)
          {
            throw new Exception("Invalid time format");
          }

        GregorianCalendar calendar = new GregorianCalendar(iYear,
                                        iMonth, iDay, iHour, iMinute);
        return (calendar.getTime());
    }

    /** method to allow execution of the SimpleReminder program */
    public static void main (String args[])
    {
        BufferedReader input =
                new BufferedReader(new InputStreamReader(System.in));
        PrintWriter output = new PrintWriter(System.out, true);

        String sReminderFile = "reminder.v1";
        if (args.length == 1)
            sReminderFile = args[0];

        SimpleReminder simpleReminder =
                    new SimpleReminder(input, output, sReminderFile);

        simpleReminder.start();
    }

}